			    BRL-CAD
			 Release 7.18.4
		       http://brlcad.org/

BRL-CAD is a powerful cross-platform open source combinatorial
Constructive Solid Geometry (CSG) solid modeling system that includes
an interactive 3D solid geometry editor, a network-distributed
symmetric multiprocessing high-performance ray-tracer,
network-distributed framebuffer support, image and signal-processing
tools, ray-tracing support for rendering and geometric analysis,
path-tracing and photon mapping support for realistic image synthesis,
animation capabilities, ray-tracing and numerical processing
libraries, a system performance analysis benchmark suite, an embedded
scripting interface, and a robust high-performance geometric
representation and analysis library.


TABLE OF CONTENTS
-----------------
  Introduction
  Table of Contents
  Getting Started
  Compile and Install
  Benchmark
  Communication
  Bug Reports/Feature Requests
  Distribution Details
  Background Information
  Future Events
  Getting Help


GETTING STARTED
----------------

This README file exists only to provide you a brief roadmap to the
contents of the BRL-CAD distribution.

Some platforms have additional platform-specific documentation that
should be consulted after reading this file:

  doc/README.MacOSX	-- Apple Mac OS X
  doc/README.IRIX	-- SGI IRIX and IRIX64
  doc/README.Linux	-- Various Linux distributions
  doc/README.Windows    -- Microsoft Windows

The information on how to install and operate the software, to become
involved with the development of the package, or to run the benchmark
performance test suite have each become sufficiently large that their
details are covered in more detail in standalone documents.  For more
details on what BRL-CAD provides, see the documentation provided on
the website at http://brlcad.org/.

Information relevant to those interested in the source code is
contained in the HACKING and COPYING files.  See doc/benchmark.tr file
for benchmark suite information.  There is a variety of other
documentation and information also available in the doc/ directory.

Please note that this distribution package does not include the
various military-specific model analysis tools such as MUVES, GIFT,
SAR, SLAVE, VAST, etc., nor does it include any military geometry
databases.  If you desire to have access to this material, please
contact one of the BRL-CAD developers.


COMPILE AND INSTALL
-------------------

See the INSTALL file for details on configuring, compiling, and
installing BRL-CAD.  In order to compile and install BRL-CAD from a
source distribution, the build tool CMake is required - see the
website at http://www.cmake.org for source and binary installers.
CMake is also available in many Linux and *BSD distributions.

Once CMake is installed, the following steps may be taken for a full
default install (starting with a source tarball):

  tar -xvzf brlcad-X.Y.Z.tar.gz
  mkdir brlcad-build
  cd brlcad-build
  cmake ../brlcad-X.Y.Z
  make
  make install

where "X.Y.Z" is the version number of BRL-CAD.  For those who prefer
a curses interface, substitute ccmake for cmake above. If the graphical
interface available from CMake is installed cmake-gui may be
substituted for cmake.  These interfaces will display the various 
configuration options available when building BRL-CAD, or the options 
may be set directly via command line -D options.
The configure wrapper script may be substituted for cmake in the above
steps for a more autotools-like configuration experience.

Users on multiprocessor systems will be able to speed up their builds
by passing the -j option to make (e.g. make -j4, assuming GNU make).

Once the build completes and assuming you have proper filesystem
permissions, you should be able to begin the installation by running
the "make install" .  By default, the package is configured to install
entirely into /usr/brlcad/.

You will need to add /usr/brlcad/bin to your path (or whatever path the
variable CMAKE_INSTALL_PREFIX was set to during configure) in order for
the BRL-CAD binaries to be in your path.


BENCHMARK
---------

After the build successfully completes, you will be able to verify
basic functionality by running the BRL-CAD Benchmark.  Running the
benchmark suite is integrated into the build system such that any time
after running CMake (in the cmake gui, configure and generate must both 
be performed) you may run:

  make benchmark

If the build is successful, you will see "CORRECT" numbers and a
performance summary at the end.  The "vgr" line effectively shows you
approximately how much faster your machine is to a VAX 11/780.  After
installing BRL-CAD, you may run the 'benchmark' tool as well to run
the BRL-CAD Benchmark.

See the paper in doc/benchmark.tr for information on analyzing and
comparing the results of the benchmark tests in more detail.


COMMUNICATION
-------------

You are invited to participate in the InterNet (aka "ARPANET")
electronic mailing list on the BRL-CAD software.  There are various
BRL-CAD mailing lists available on the project website at
http://sourceforge.net/mail/?group_id=105292 including three
developer-oriented lists, a user list, and a read-only news
announcement list.  See the posted archives to get a feel for the
volume of list traffic for each list and to read/search past
discussions.

On-line web forums are likewise available on the project site at
http://sourceforge.net/forum/?group_id=105292

A variety of information on BRL-CAD is available via the
World-Wide-Web.  Papers, documentation, and the software repository
may be found at:

  http://brlcad.org

In addition to the above, there is an official BRL-CAD IRC channel on
the Freenode network that may be reached by joining #brlcad on
irc.freenode.net connecting via port 6667.  For first-time users,
there is a web-based IRC client available for certain browsers at:

  http://irc.brlcad.org

IRC is the preferred method of communication to interact with the
BRL-CAD developers, to get involved with the project, obtain support,
or provide direct feedback.


BUG REPORTS AND FEATURE REQUESTS
--------------------------------

Please report any bugs encountered to the project bug tracker at
http://sourceforge.net/tracker/?group_id=105292&atid=640802

Similarly, please post any request for feature enhancements or support
request to http://sourceforge.net/tracker/?group_id=105292&atid=640805
and http://sourceforge.net/tracker/?group_id=105292&atid=640803
respectively.


DISTRIBUTION DETAILS
--------------------

BRL-CAD is freely available and distributed in both binary and source
code form through the BRL-CAD project website on Sourceforge:

  http://sf.net/projects/brlcad/

The package is similarly available via a Subversion repository.  To
obtain the latest, follow the directions for anonymous checkout at:

  http://sourceforge.net/svn/?group_id=105292

There is also a paid-for full-service support distribution available
for those interested through SURVICE Engineering, Inc., and the
Survivability/Vulnerability Information Analysis Center (SURVIAC).

SURVICE provides their own BRL-CAD distribution that comes with
commercial support, documentation, and installation media.  Full
technical assistance is provided to their commercial distributions by
phone, FAX, postal mail, or e-mail.  Agencies of the Federal
Government may acquire the full service distribution with a MIPR or
OGA funds transfer.

For further details, contact:

BRL-CAD Distribution
SURVIAC Aberdeen Satellite Office
4695 Millennium Drive
Belcamp, MD 210017-1505  USA


BACKGROUND INFORMATION
----------------------

BRL-CAD is the primary tri-service solid modeling CAD system used by
the U.S. military to model weapons systems for vulnerability and
lethality analyses.  The solid modeling system is frequently used in a
wide range of military and industrial application including in the
design and analysis of vehicles, mechanical parts, and architecture.
The package has also been used in radiation dose planning, medical
visualization, computer graphics education, CSG concepts and modeling
education, and system performance benchmark testing among other
purposes.

Actively developed since 1979, BRL-CAD is a collection of more than
400 tools, utilities, and applications comprising more than a million
lines of source code.  BRL-CAD supports a great variety of geometric
representations including an extensive set of traditional CSG
primitive implicit solids such as boxes, ellipsoids, cones, and tori,
as well as explicit solids made from closed collections of Uniform
B-Spline Surfaces, Non-Uniform Rational B-Spline (NURBS) surfaces,
n-Manifold Geometry (NMG), and purely faceted mesh geometry.  All
geometric objects may be combined using boolean set-theoretic CSG
operations including union, intersection, and difference.

BRL-CAD has been under active development for more than 20 years with
a portability heritage that includes systems such as a DEC VAX-11/780
running 4.3 BSD; DECStations running ULTRIX; Silicon Graphics 3030, 4D
"IRIS", O2, Onyx, and Origin systems running various versions of IRIX;
Sun Microsystems Sun-3 and Sun-4 Sparcs running SunOS; the Cray 1,
Cray X-MP, Cray Y-MP, and Cray 2 running UNICOS; DEC Alpha AXP running
OSF/1; Apple Macintosh II running A/UX; iPSC/860 Hypercube running
NX/2; the Alliant FX/8, FX/80, and FX/2800; Gould/Encore SEL PowerNode
6000/9000 and NP1; NeXT workstations; IBM RS/6000; HPPA 9000/700
running HPUX; Ardent/Stardent; Encore Multi-Max; and much much more.
BRL-CAD's extensive portability of course also includes a plethora of
common desktop and server versions of BSD, IRIX, Solaris, Mac OS X,
Linux, and Windows.

The BRL-CAD package is intentionally designed to be extensively
cross-platform and is actively developed on and maintained for many
common operating system environments including for BSD, IRIX, Linux,
Solaris, Mac OS X, and Windows among others.  The entire package is
distributed in binary and source code form as free open source
software (FOSS), provided under Open Source Initiative (OSI) approved
license terms.  BRL-CAD was released as open source in December 2004
with portions licensed under the GPL, LGPL, GFDL, and BSD licenses.
See the COPYING file for more details.


FUTURE EVENTS
-------------

It is expected that new releases of BRL-CAD will be issued
approximately once a month.  A new release is scheduled for
distribution at the beginning of every month, usually being posted
sometime within the first week of every month.

Depending on need, demand, and activity, additional releases may also
be made.  The additional releases will be heavily dependant upon the
level of user involvement and contributions in particular.
Information about new releases will be routinely provided by
electronic mail to recipients of this software as well as to the
BRL-CAD NEWS mailing list:

  http://sourceforge.net/mail/?group_id=105292

Periodically, training and user group meetings are provided with
announcements similarly posted to the BRL-CAD NEWS mailing list.
Several BRL-CAD developers also regularly attend the ACM Siggraph
conference, the ACM Solid and Physical Modeling conference, and other
similar conferences, conventions, and symposiums as well.


GETTING HELP
------------

Feel free to direct any unanswered questions relating to BRL-CAD via
e-mail to 'devs@brlcad.org' or to any of the other contact means
described above in the COMMUNICATIONS section of this document.

Best Wishes,
The BRL-CAD Development Team
devs@brlcad.org
